'use strict';

const { join } = require('path');
const { name, version } = require(join(__dirname, 'package.json'));
const compilePostCSS = require('broccoli-postcss-single');
const { WatchedDir } = require('broccoli-source');
const CleanCSS = require('broccoli-clean-css');
const mergeTrees = require('broccoli-merge-trees');
const createFile = require('broccoli-file-creator');

module.exports = {
  name,

  hintingEnabled() {
    return false;
  },

  shouldIncludeChildAddon(addon) {
    // WARN: don't infinitely recurse - it's the dummy test app depends on dummy-addon, not addon itself
    return addon.name.indexOf('dummy') === -1;
  },

  treeForPublic() {
    if (this.app === undefined) return null;

    const isAddon = this.parent.isEmberCLIAddon();
    const tailwindBase = isAddon
      ? join(this.parent.root, 'tests', 'dummy', 'tailwindcss')
      : join(this.parent.root, 'tailwindcss');
    return this.buildTailwindNode(tailwindBase, isAddon);
  },

  buildTailwindNode(tailwindBase, isAddon) {
    const plugins = this.pluginsForTailwind(tailwindBase, isAddon);

    const tailwindNode = compilePostCSS(
      new WatchedDir(tailwindBase, { annotation: 'WatchingTailwindCSS' }),
      'index.css',
      'assets/tailwind.css',
      { annotation: 'TailwindCSS', cacheInclude: [/.*\.(css|js)$/], plugins }
    );

    return this.app.isProduction ? new CleanCSS(tailwindNode) : tailwindNode;
  },

  pluginsForTailwind(tailwindBase, isAddon) {
    const isProduction = this.app.env === 'production';
    const plugins = [require('tailwindcss')(join(tailwindBase, 'tailwind.js'))];

    if (isProduction) {
      const addonRoot = isAddon ? this.root : this.parent.root;
      const appRoot = this.parent.root;
      const appBase = isAddon ? join(appRoot, 'tests', 'dummy') : appRoot;
      const content = [
        join(appBase, 'app', 'index.html'),
        join(appBase, 'app', '**', '*.js'),
        join(appBase, 'app', '**', '*.hbs'),
        join(addonRoot, 'addon', '**', '*.js'),
        join(addonRoot, 'addon', '**', '*.hbs'),
      ];
      const extractor = (content) => content.match(/[A-Za-z0-9-_:/]+/g) || [];
      const whitelistPatterns = [/^_/];

      plugins.push({
        module: require('@fullhuman/postcss-purgecss'),
        options: { content, defaultExtractor: extractor, whitelistPatterns },
      });
    }

    return plugins;
  },

  contentFor(type, config, content) {
    if (type === 'head') {
      return [
        ...content.filter((entry) => entry && entry.length > 0),
        `<link integrity="" rel="stylesheet" href="${config.rootURL}assets/tailwind.css" />`,
      ];
    }
  },

  treeForVendor(vendorTree) {
    const content = `Ember.libraries.register('${name}', '${version}');`;
    const versionNode = createFile(`${name}/register-version.js`, content);
    return mergeTrees([vendorTree, versionNode], {
      annotation: `${name}: Version`,
      overwrite: true,
    });
  },

  included() {
    this._super.included.apply(this, arguments);
    this.import(`vendor/${name}/register-version.js`);
  },
};
